@@ -3,11 +3,13 @@
 
 #include <sys/types.h>
 #include <stdbool.h>
+#include <libHX/string.h>
 #include "compiler.h"
 
 #define __STRINGIFY_EXPAND(s) #s
 #define __STRINGIFY(s)        __STRINGIFY_EXPAND(s)
 
+struct HXbtree;
 struct config;
 struct vol;
 
