@@ -94,7 +94,6 @@ static void log_output(int fd, const char *cmsg)
 		} while (fgets(buf, sizeof(buf), fp) != NULL);
 	}
 	fclose(fp);
-	return;
 }
 
 /**
@@ -114,7 +113,7 @@ static void run_lsof(const struct config *const config,
 	unsigned int i;
 	pid_t pid;
 
-	if (stat(HXbtree_get(vinfo, "MNTPT"), &sb) < 0 && errno == EEXIST)
+	if (stat(HXbtree_get(vinfo, "MNTPT"), &sb) < 0 && errno == ENOENT)
 		return;
 
 	if (config->command[CMD_LSOF][0] == NULL)
@@ -132,7 +131,6 @@ static void run_lsof(const struct config *const config,
 	if (waitpid(pid, NULL, 0) < 0)
 		l0g("error waiting for child: %s\n", strerror(errno));
 	spawn_restore_sigchld();
-	return;
 }
 
 
@@ -322,7 +320,6 @@ static void vol_to_dev(char *match, size_t s, const struct vol *vol)
 		match[s-1] = '\0';
 		break;
 	}
-	return;
 }
 
 #if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__APPLE__)
@@ -379,7 +376,6 @@ static void log_pm_input(const struct config *const config,
 	w4rn("use_fstab:     %d\n", vpt->use_fstab);
 	w4rn("----------------------\n");
 	hmc_free(options);
-	return;
 }
 
 /**
@@ -511,6 +507,9 @@ int do_unmount(const struct config *config, struct vol *vpt,
 		run_lsof(config, vinfo);
 
 	switch (vpt->type) {
+		case CMD_CRYPTMOUNT:
+			type = CMD_CRYPTUMOUNT;
+			break;
 		case CMD_SMBMOUNT:
 			type = CMD_SMBUMOUNT;
 			break;
@@ -534,15 +533,6 @@ int do_unmount(const struct config *config, struct vol *vpt,
 	for (i = 0; config->command[type][i] != NULL; ++i)
 		add_to_argv(_argv, &_argc, config->command[type][i], vinfo);
 
-	/*
-	 * FIXME: ugly hack to support umount.crypt script. I hope that
-	 * util-linux will have native dm_crypt support some day.
-	 */
-	if (vpt->type == CMD_CRYPTMOUNT) {
-		_argc = 0;
-		add_to_argv(_argv, &_argc, "/sbin/umount.crypt", vinfo);
-		add_to_argv(_argv, &_argc, "%(MNTPT)", vinfo);
-	}
 	log_argv(_argv);
 	if (!spawn_start(_argv, &pid, NULL, NULL, &cstderr, set_myuid, NULL)) {
 		ret = 0;
@@ -714,6 +704,15 @@ static int check_filesystem(const struct config *config, const struct vol *vpt,
 	assert(password_len >= 0 &&
 	       password_len <= MAX_PAR + EVP_MAX_BLOCK_LENGTH);
 
+	if (vpt->type == CMD_CRYPTMOUNT)
+		/*
+		 * Cryptmount involves dm-crypt or LUKS, so using the raw
+		 * device as fsck target is meaningless.
+		 * So we do _not_ set FSCKTARGET in vinfo at all, and
+		 * mount_set_fsck() depends on this behavior.
+		 */
+		return 0;
+
 	fsck_target = vpt->volume;
 
 	if (config->command[CMD_FSCK][0] == NULL) {
@@ -735,8 +734,6 @@ static int check_filesystem(const struct config *config, const struct vol *vpt,
 		w4rn("volume not a loopback (options: %s)\n", options);
 		hmc_free(options);
 	}
-	/* FIXME: NEW */
-	/* FIXME: need to fsck /dev/mapper/whatever... */
 	format_add(vinfo, "FSCKTARGET", fsck_target);
 	for (i = 0; config->command[CMD_FSCK][i]; ++i)
 		add_to_argv(_argv, &_argc, config->command[CMD_FSCK][i], vinfo);
@@ -766,6 +763,36 @@ static int check_filesystem(const struct config *config, const struct vol *vpt,
 #endif
 }
 
+/**
+ * mount_set_fsck - set the FSCK environment variable for mount.crypt
+ * @config:	configuration
+ * @vol:	current volume
+ * @vinfo:	variable substituions
+ */
+static void mount_set_fsck(const struct config *config,
+    const struct vol *vol, struct HXbtree *vinfo)
+{
+	hmc_t *string, *current;
+	unsigned int i;
+
+	if (vol->type != CMD_CRYPTMOUNT)
+		return;
+
+	format_add(vinfo, "FSCKTARGET", "");
+	string = hmc_sinit("");
+	for (i = 0; config->command[CMD_FSCK][i] != NULL; ++i) {
+		if (HXformat_aprintf(vinfo, &current,
+		    config->command[CMD_FSCK][i]) > 0) {
+			hmc_strcat(&string, current);
+			hmc_strcat(&string, " ");
+		}
+		hmc_free(current);
+	}
+
+	setenv("FSCK", string, true);
+	hmc_free(string);
+}
+
 /**
  * do_mount -
  * @config:	current config
@@ -866,6 +893,8 @@ int do_mount(const struct config *config, struct vol *vpt,
 	/* send password down pipe to mount process */
 	if (vpt->type == CMD_SMBMOUNT || vpt->type == CMD_CIFSMOUNT)
 		setenv("PASSWD_FD", "0", 1);
+
+	mount_set_fsck(config, vpt, vinfo);
 	log_argv(_argv);
 	mount_user = strcmp(vpt->fstype, "fuse") == 0 ?
 	             vpt->user : NULL;
@@ -892,8 +921,7 @@ int do_mount(const struct config *config, struct vol *vpt,
 
 	spawn_restore_sigchld();
 	if (Debug)
-		if (system("df -Ta") < 0)
-			;
+		spawn_synchronous((const char *const []){"df", "-Ta", NULL});
 
 	/* pass on through the result from the umount process */
 	return !WEXITSTATUS(child_exit);
