@@ -14,6 +14,7 @@
 
 #include <security/pam_appl.h>
 #include <security/pam_modules.h>
+#include <sys/mman.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <assert.h>
@@ -92,7 +93,6 @@ static void parse_pam_args(int argc, const char **argv)
 		else
 			w4rn("bad pam_mount option \"%s\"\n", argv[i]);
 	}
-	return;
 }
 
 /**
@@ -108,7 +108,6 @@ static void clean_config(pam_handle_t *pamh, void *data, int err)
 {
 	w4rn("Clean global config (%d)\n", err);
 	freeconfig(data);
-	return;
 }
 
 /**
@@ -124,14 +123,14 @@ static void clean_config(pam_handle_t *pamh, void *data, int err)
  */
 static void clean_system_authtok(pam_handle_t *pamh, void *data, int errcode)
 {
-	w4rn("clean system authtok (%d)\n", errcode);
-/*
+	w4rn("clean system authtok=%p (%d)\n", data, errcode);
+
 	if (data != NULL) {
-		memset(data, 0, strlen(data));
+		unsigned int len = strlen(data) + 1;
+		memset(data, 0, len);
+		munlock(data, len);
 		free(data);
 	}
-*/
-	return;
 }
 
 /**
@@ -304,13 +303,15 @@ PAM_EXTERN EXPORT_SYMBOL int pam_sm_authenticate(pam_handle_t *pamh, int flags,
 		ret = PAM_AUTH_ERR;
 		goto out;
 	}
-	w4rn("saving authtok for session code\n");
+	w4rn("saving authtok for session code (authtok=%p)\n", authtok);
 	ret = pam_set_data(pamh, "pam_mount_system_authtok", authtok,
 	                   clean_system_authtok);
 	if (ret != PAM_SUCCESS) {
 		l0g("error trying to save authtok for session code\n");
 		goto out;
 	}
+	if (mlock(authtok, strlen(authtok) + 1) < 0)
+		w4rn("mlock authtok: %s\n", strerror(errno));
 	assert(ret != PAM_SUCCESS ||
 	       pam_get_data(pamh, "pam_mount_system_authtok", &tmp) ==
 	       PAM_SUCCESS);
@@ -331,7 +332,6 @@ static void envpath_init(const char *new_path)
 {
 	envpath_saved = getenv("PATH");
 	setenv("PATH", new_path, true);
-	return;
 }
 
 static void envpath_restore(void)
@@ -340,7 +340,6 @@ static void envpath_restore(void)
 		unsetenv("PATH");
 	else
 		setenv("PATH", envpath_saved, true);
-	return;
 }
 
 /**
