@@ -25,13 +25,13 @@ enum command_type {
 	CMD_SMBMOUNT,
 	CMD_SMBUMOUNT,
 	CMD_CIFSMOUNT,
-	CMD_DAVMOUNT,
 	CMD_NCPMOUNT,
 	CMD_NCPUMOUNT,
 	CMD_FUSEMOUNT,
 	CMD_FUSEUMOUNT,
 	CMD_LCLMOUNT,
 	CMD_CRYPTMOUNT,
+	CMD_CRYPTUMOUNT,
 	CMD_NFSMOUNT,
 	CMD_UMOUNT,
 	CMD_PMHELPER,
