@@ -49,6 +49,11 @@ enum {
 	CMDA_PATH,
 };
 
+enum {
+	OPT_TREE_FLAGS =
+		HXBT_MAP | HXBT_CKEY | HXBT_SCMP | HXBT_CID,
+};
+
 struct callbackmap {
 	const char *name;
 	const char *(*func)(xmlNode *, struct config *, unsigned int);
@@ -69,6 +74,7 @@ static int rc_volume_cond_ext(const struct passwd *, xmlNode *);
 /* Variables */
 static const struct callbackmap cf_tags[];
 static const struct pmt_command default_command[];
+static bool onetime_options_allow, onetime_options_require;
 
 //-----------------------------------------------------------------------------
 /**
@@ -129,14 +135,80 @@ void freeconfig(struct config *config)
 	free(config->msg_authpw);
 	free(config->msg_sessionpw);
 	free(config->path);
-	return;
+}
+
+/**
+ * str_to_optlist -
+ * @optlist:	destination list
+ * @str:	string to parse
+ *
+ * Break down @str into its option. This function modifies @str in-place.
+ * This is ok, since it is already an allocated string (i.e. does belong to
+ * pam_mount). Caller frees it anyway right away.
+ */
+static bool str_to_optkv(struct HXclist_head *optlist, char *str)
+{
+	char *value, *ptr;
+	struct kvp *kvp;
+
+	if (str == NULL || *str == '\0')
+		return true;
+
+	while ((ptr = HX_strsep(&str, ",")) != NULL) {
+		kvp = xmalloc(sizeof(struct kvp));
+		if (kvp == NULL)
+			return false;
+		HXlist_init(&kvp->list);
+		value = strchr(ptr, '=');
+		if (value != NULL) {
+			*value++ = '\0';
+			kvp->key   = xstrdup(ptr);
+			kvp->value = xstrdup(value);
+			if (kvp->key == NULL || kvp->value == NULL)
+				goto out;
+			HXclist_push(optlist, &kvp->list);
+		} else {
+			kvp->key = xstrdup(ptr);
+			if (kvp->key == NULL)
+				goto out;
+			kvp->value = NULL;
+			HXclist_push(optlist, &kvp->list);
+		}
+	}
+
+	return true;
+ out:
+	free(kvp->key);
+	free(kvp->value);
+	free(kvp);
+	return false;
+}
+
+static bool str_to_optlist(struct HXbtree *optlist, char *str)
+{
+	char *value, *ptr;
+
+	if (str == NULL || *str == '\0')
+		return true;
+
+	while ((ptr = HX_strsep(&str, ",")) != NULL) {
+		value = strchr(ptr, '=');
+		if (value != NULL) {
+			*value++ = '\0';
+			HXbtree_add(optlist, ptr, value);
+		} else {
+			HXbtree_add(optlist, ptr, NULL);
+		}
+	}
+
+	return true;
 }
 
 void initconfig(struct config *config)
 {
 	unsigned int i, j;
-	static const unsigned int flags =
-		HXBT_MAP | HXBT_CKEY | HXBT_SCMP | HXBT_CID;
+	char options_allow[]   = "nosuid,nodev";
+	char options_require[] = "nosuid,nodev";;
 
 	memset(config, 0, sizeof(*config));
 	config->debug      = true;
@@ -154,12 +226,12 @@ void initconfig(struct config *config)
 			config->command[default_command[i].type][j] =
 				xstrdup(default_command[i].def[j]);
 
-	config->options_allow   = HXbtree_init(flags);
-	config->options_require = HXbtree_init(flags);
-	config->options_deny    = HXbtree_init(flags);
-
+	config->options_allow   = HXbtree_init(OPT_TREE_FLAGS);
+	config->options_require = HXbtree_init(OPT_TREE_FLAGS);
+	config->options_deny    = HXbtree_init(OPT_TREE_FLAGS);
+	str_to_optlist(config->options_allow, options_allow);
+	str_to_optlist(config->options_require, options_require);
 	HXclist_init(&config->volume_list);
-	return;
 }
 
 bool readconfig(const char *file, bool global_conf, struct config *config)
@@ -208,7 +280,7 @@ static char *expand_home(const char *user, char *path, size_t size)
 	struct passwd *pe;
 	char *buf;
 
-	if (*path != '~')
+	if (path[0] != '~' || path[1] != '/')
 		return path;
 
 	if ((pe = getpwnam(user)) == NULL) {
@@ -559,73 +631,6 @@ static const char *rc_mkmountpoint(xmlNode *node, struct config *config,
 	return NULL;
 }
 
-/**
- * str_to_optlist -
- * @optlist:	destination list
- * @str:	string to parse
- *
- * Break down @str into its option. This function modifies @str in-place.
- * This is ok, since it is already an allocated string (i.e. does not
- * belong to libxml but to pam_mount). Caller frees it anyway right away.
- */
-static bool str_to_optkv(struct HXclist_head *optlist, char *str)
-{
-	char *value, *ptr;
-	struct kvp *kvp;
-
-	if (str == NULL || *str == '\0')
-		return true;
-
-	while ((ptr = HX_strsep(&str, ",")) != NULL) {
-		kvp = xmalloc(sizeof(struct kvp));
-		if (kvp == NULL)
-			return false;
-		HXlist_init(&kvp->list);
-		value = strchr(ptr, '=');
-		if (value != NULL) {
-			*value++ = '\0';
-			kvp->key   = xstrdup(ptr);
-			kvp->value = xstrdup(value);
-			if (kvp->key == NULL || kvp->value == NULL)
-				goto out;
-			HXclist_push(optlist, &kvp->list);
-		} else {
-			kvp->key = xstrdup(ptr);
-			if (kvp->key == NULL)
-				goto out;
-			kvp->value = NULL;
-			HXclist_push(optlist, &kvp->list);
-		}
-	}
-
-	return true;
- out:
-	free(kvp->key);
-	free(kvp->value);
-	free(kvp);
-	return false;
-}
-
-static bool str_to_optlist(struct HXbtree *optlist, char *str)
-{
-	char *value, *ptr;
-
-	if (str == NULL || *str == '\0')
-		return true;
-
-	while ((ptr = HX_strsep(&str, ",")) != NULL) {
-		value = strchr(ptr, '=');
-		if (value != NULL) {
-			*value++ = '\0';
-			HXbtree_add(optlist, ptr, value);
-		} else {
-			HXbtree_add(optlist, ptr, NULL);
-		}
-	}
-
-	return true;
-}
-
 static const char *rc_mntoptions(xmlNode *node, struct config *config,
     unsigned int command)
 {
@@ -636,6 +641,11 @@ static const char *rc_mntoptions(xmlNode *node, struct config *config,
 		return "Tried to set <mntoptions allow=...> from user config";
 
 	if ((options = xmlGetProp_2s(node, "allow")) != NULL) {
+		if (!onetime_options_allow) {
+			HXbtree_free(config->options_allow);
+			config->options_allow = HXbtree_init(OPT_TREE_FLAGS);
+			onetime_options_allow = true;
+		}
 		ret = str_to_optlist(config->options_allow, options);
 		free(options);
 		if (!ret)
@@ -650,6 +660,11 @@ static const char *rc_mntoptions(xmlNode *node, struct config *config,
 	}
 
 	if ((options = xmlGetProp_2s(node, "require")) != NULL) {
+		if (!onetime_options_require) {
+			HXbtree_free(config->options_require);
+			config->options_require = HXbtree_init(OPT_TREE_FLAGS);
+			onetime_options_require = true;
+		}
 		ret = str_to_optlist(config->options_require, options);
 		free(options);
 		if (!ret)
@@ -729,7 +744,7 @@ static int rc_volume_cond_and(const struct passwd *pwd, xmlNode *node)
  */
 static int rc_volume_cond_or(const struct passwd *pwd, xmlNode *node)
 {
-	unsigned int count;
+	unsigned int count = 0;
 	int ret;
 
 	for (node = node->children; node != NULL; node = node->next) {
@@ -794,7 +809,7 @@ static int rc_volume_cond_xor(const struct passwd *pwd, xmlNode *node)
 static int rc_volume_cond_not(const struct passwd *pwd, xmlNode *node)
 {
 	unsigned int count = 0;
-	bool ret;
+	bool ret = true;
 
 	for (node = node->children; node != NULL; node = node->next) {
 		if (node->type != XML_ELEMENT_NODE)
@@ -1028,6 +1043,9 @@ static int rc_volume_cond_simple(const struct passwd *pwd, xmlNode *node)
 		else if (pwd->pw_uid == 0 || strcmp(pwd->pw_name, "root") == 0)
 			/* The wildcard never matches root */
 			for_me &= false;
+	} else if (pwd->pw_uid == 0 || strcmp(pwd->pw_name, "root") == 0) {
+		/* No user field at all generally implies the wildcard */
+		for_me &= false;
 	}
 	if (uid != NULL) {
 		ret = __rc_volume_cond_id(uid, pwd->pw_uid);
@@ -1048,14 +1066,13 @@ static int rc_volume_cond_simple(const struct passwd *pwd, xmlNode *node)
 		for_me &= ret;
 	}
 	if (sgrp != NULL) {
-		ret = __rc_volume_cond_pgrp(sgrp, pwd->pw_gid, false);
+		bool ret2 = __rc_volume_cond_pgrp(sgrp, pwd->pw_gid, false);
 		if (ret < 0)
 			goto out;
-		for_me &= ret;
 		ret = user_in_sgrp(pwd->pw_name, sgrp, false);
 		if (ret < 0)
 			goto out;
-		for_me &= ret;
+		for_me &= ret || ret2;
 	}
 	if (invert != NULL) {
 		l0g("The \"invert\" attribute is deprecated, support will "
@@ -1231,7 +1248,6 @@ static const struct pmt_command default_command[] = {
 	{CMD_SMBMOUNT,   "smbfs", "smbmount",   {"smbmount", "//%(SERVER)/%(VOLUME)", "%(MNTPT)", "-o", "username=%(USER),uid=%(USERUID),gid=%(USERGID)%(before=\",\" OPTIONS)", NULL}},
 	{CMD_SMBUMOUNT,  "smbfs", "smbumount",  {"smbumount", "%(MNTPT)", NULL}},
 	{CMD_CIFSMOUNT,  "cifs",  "cifsmount",  {"mount", "-t", "cifs", "//%(SERVER)/%(VOLUME)", "%(MNTPT)", "-o", "username=%(USER),uid=%(USERUID),gid=%(USERGID)%(before=\",\" OPTIONS)", NULL}},
-	{CMD_DAVMOUNT,   "davfs", "davmount",   {"mount", "-t", "davfs", "%(SERVER)/%(VOLUME)", "%(MNTPT)", "-o", "username=%(USER),uid=%(USERUID),gid=%(USERGID)%(before=\",\" OPTIONS)", NULL}},
 	{CMD_NCPMOUNT,   "ncpfs", "ncpmount",   {"ncpmount", "%(SERVER)/%(USER)", "%(MNTPT)", "-o", "pass-fd=0,volume=%(VOLUME)%(before=\",\" OPTIONS)", NULL}},
 	{CMD_NCPUMOUNT,  "ncpfs", "ncpumount",  {"ncpumount", "%(MNTPT)", NULL}},
 	{CMD_FUSEMOUNT,  "fuse",  "fusemount",  {"mount.fuse", "%(VOLUME)", "%(MNTPT)", "%(before=\"-o \" OPTIONS)", NULL}},
@@ -1241,6 +1257,7 @@ static const struct pmt_command default_command[] = {
 	{CMD_LCLMOUNT,   NULL,    "lclmount",   {"mount", "-p0", "-t", "%(FSTYPE)", "%(VOLUME)", "%(MNTPT)", "%(before=\"-o\" OPTIONS)", NULL}},
 	/* Hope to have this in util-linux (LCLMOUNT) some day: */
 	{CMD_CRYPTMOUNT, "crypt", "cryptmount", {"mount.crypt", "%(before=\"-o \" OPTIONS)", "%(VOLUME)", "%(MNTPT)", NULL}},
+	{CMD_CRYPTUMOUNT, "crypt", "cryptumount", {"umount.crypt", "%(MNTPT)", NULL}},
 	{CMD_UMOUNT,     NULL,    "umount",     {"umount", "%(MNTPT)", NULL}},
 	{CMD_LSOF,       NULL,    "lsof",       {"lsof", "%(MNTPT)", NULL}},
 	/*
@@ -1262,7 +1279,6 @@ static const struct pmt_command default_command[] = {
 static const struct callbackmap cf_tags[] = {
 	{"cifsmount",       rc_command,             CMD_CIFSMOUNT},
 	{"cryptmount",      rc_command,             CMD_CRYPTMOUNT},
-	{"davmount",        rc_command,             CMD_DAVMOUNT},
 	{"debug",           rc_debug,               CMD_NONE},
 	{"fd0ssh",          rc_command,             CMD_FD0SSH},
 	{"fsckloop",        rc_fsckloop,            CMD_NONE},
