@@ -184,8 +184,7 @@ bool volume_record_sane(const struct config *config, const struct vol *vpt)
 		return false;
 	}
 	if (vpt->type == CMD_SMBMOUNT || vpt->type == CMD_CIFSMOUNT ||
-	    vpt->type == CMD_NCPMOUNT || vpt->type == CMD_NFSMOUNT ||
-	    vpt->type == CMD_DAVMOUNT)
+	    vpt->type == CMD_NCPMOUNT || vpt->type == CMD_NFSMOUNT)
 	    	if (strlen(vpt->server) == 0) {
 			l0g("remote mount type specified without server\n");
 			return false;
